/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2018 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    ER_Demo.c
 *      Purpose: RTX ER demo example program
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>

#include "RTE_Components.h"
#include  CMSIS_device_header
#include "cmsis_os2.h"

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#include <math.h>
#include <stdlib.h>

#define TABLE_SIZE 1000
float sin_table[TABLE_SIZE];
float cos_table[TABLE_SIZE];

typedef struct {                       
  float num; 
	int output;	
} MyType;
 
MyType volatile mysymbol;  

osThreadId_t tid_CalcSinTable;                    // Thread id of thread: CalcSinTable
osThreadId_t tid_CalcCosTable;                    // Thread id of thread: CalcCosTable
osThreadId_t tid_Prime;                           // Thread id of thread: Prime

/*----------------------------------------------------------------------------
 *      Thread 1 Sin table calculation
 *---------------------------------------------------------------------------*/
void CalcSinTable (void *argument) {
  for (;;) {
		unsigned int i, max_i;
		float f = 0.0;  
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever); // wait for an event flag 0x0001
    	
		max_i = TABLE_SIZE - (rand () % 500);
#ifdef RTE_Compiler_EventRecorder
		EventStartBv (0, max_i, 0); 	            // Start group B, slot 0, pass size information
#endif
		for (i = 0; i < max_i; i++)  {
  		sin_table[i] = sinf(f);
	  	f = f + (3.141592 / TABLE_SIZE);
    }
#ifdef RTE_Compiler_EventRecorder
		EventStopBv (0, max_i, 0);                  //  Stop group B, slot 0, pass size information
#endif
		osThreadFlagsSet(tid_CalcCosTable, 0x0001); // Set flag for next thread: CalcCosTable
	  osDelay(500); 
	}
}

/*----------------------------------------------------------------------------
 *      Thread 2  Cos table calculation
 *---------------------------------------------------------------------------*/
void CalcCosTable (void *argument) {
  for (;;) {
    unsigned int i, max_i;
    float f = 0.0;
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever); // wait for an event flag 0x0001
  
    max_i = TABLE_SIZE - (rand () % 500);
#ifdef RTE_Compiler_EventRecorder
		EventStartCv(0, max_i, 0); 	              // Start group C, slot 0, pass size information
#endif
    for (i = 0; i < max_i; i++)  {
      cos_table[i] = cosf(f);
      f = f + (3.141592 / TABLE_SIZE);	
      mysymbol.num = f;                           // output the value of f to Component Viewer
		}
#ifdef RTE_Compiler_EventRecorder
    EventStopCv(0, max_i, 0);                     // Stop group C, slot 0, pass size information
#endif
    osThreadFlagsSet(tid_Prime, 0x0001);          // Set flag for next thread: Prime
    osDelay(500); 
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 Prime number calculation
 *---------------------------------------------------------------------------*/
void Prime (void *argument) {
  for (;;) {
		int i, j, NumberIsPrime, max_N;
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever); // wait for an event flag 0x0001

    max_N = TABLE_SIZE - (rand () % 500);
    mysymbol.output =max_N;                       // output the value of i to Component Viewer
#ifdef RTE_Compiler_EventRecorder
    EventStartD(0);
#endif
    for(i = 3; i <= max_N; i++){
      NumberIsPrime = 1;
      if ((i & 0x01)!=1) {                        // Every even i can be omitted from the subsequent calculation
        continue;                                 // Continue with next i
      }
      for(j = 2; j <= i/2; j++){                  // Check if any number between 2 to i/2 divides j completely
        if(i % j == 0){                           // If yes, i cannot be a prime number
          NumberIsPrime = 0;
          break;
        }
      }
      osDelay(10);
#ifdef RTE_Compiler_EventRecorder
      if(NumberIsPrime==1 && max_N != 1) {        // If the number is prime and not '1',
         EventRecord2 (0x0A01, i, 0);              // Display prime number in Event Recorder window
      }
#endif
      }  
#ifdef RTE_Compiler_EventRecorder
      EventStopD(0);
#endif
      osThreadFlagsSet(tid_CalcSinTable, 0x0001); // Set flag for next thread: Led
      osDelay(500);
  }
}

/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {
	tid_CalcSinTable = osThreadNew(CalcSinTable, NULL, NULL);
	tid_CalcCosTable = osThreadNew(CalcCosTable, NULL, NULL);
	tid_Prime        = osThreadNew(Prime,        NULL, NULL);
	osThreadFlagsSet(tid_CalcSinTable, 0x0001);   // Set flag for next thread: CalcSinTable thread
	osDelay(osWaitForever);
}

int main (void) {

  SystemCoreClockUpdate();
#ifdef RTE_Compiler_EventRecorder
  EventRecorderInitialize(EventRecordAll, 1U);    // Initialize and start Event Recorder
#endif 
  osKernelInitialize();                           // Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, NULL);              // Create application main thread
  osKernelStart();                                // Start thread execution
  osDelay(osWaitForever);
}
